movie <- read.table("/Users/Thierry/Documents/R/Data/SY19/movie.data",header=TRUE)

library('splines')
attach(movie)
n<-nrow(movie)

#----------------------------------
# Q1
#----------------------------------
plot(log(BUDGET),log(BOX),xlab="log(BUDGET)",ylab="log(BOX)")
plot(BUZZ,log(BOX),xlab="BUZZ",ylab="log(BOX)")
plot(STARPOWR,log(BOX),xlab="STARPOWR",ylab="log(BOX)")
boxplot(log(BOX) ~ ACTION)
boxplot(log(BOX) ~ MPRATING)


#----------------------------------
# Q2
#----------------------------------
# Polynomial regression
# Budget

reg<-lm(log(BOX) ~ poly(log(BUDGET),2),data=movie)  # degree 2
plot(log(BUDGET),log(BOX))
xt<-seq(min(BUDGET),max(BUDGET),1)
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)


reg<-lm(log(BOX) ~ poly(log(BUDGET),3),data=movie)  # degree 3
plot(log(BUDGET),log(BOX))
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)


# Cross validation

# 10-fold cross-validation
K<-10
folds=sample(1:K,n,replace=TRUE)
degree<-c(1,2,3,4)
N<-length(degree)
CV<-rep(0,N)
for(i in (1:N)){
  for(k in (1:K)){
    print(c(i,k))
    reg<-lm(log(BOX) ~ poly(log(BUDGET),degree[i]),data=movie[folds!=k,])
    pred<-predict(reg,newdata=movie[folds==k,])
    CV[i]<-CV[i]+ sum((pred-log(BOX[folds==k]))^2)
  }
  CV[i]<-CV[i]/n
}
plot(degree[1:3],CV[1:3],type='b')

# Buzz

reg<-lm(log(BOX) ~ poly(BUZZ,2),data=movie)  # degree 2
plot(BUZZ,log(BOX))
xt<-seq(min(BUZZ),max(BUZZ),0.1)
yh<-predict(reg,newdata=data.frame(BUZZ=xt))
lines(xt,yh)


reg<-lm(log(BOX) ~ poly(BUZZ,3),data=movie)  # degree 3
plot(BUZZ,log(BOX))
yh<-predict(reg,newdata=data.frame(BUZZ=xt))
lines(xt,yh)


# Cross validation

# 10-fold cross-validation
K<-10
folds=sample(1:K,n,replace=TRUE)
degree<-c(1,2,3,4)
N<-length(degree)
CV<-rep(0,N)
for(i in (1:N)){
  for(k in (1:K)){
    print(c(i,k))
    reg<-lm(log(BOX) ~ poly(BUZZ,degree[i]),data=movie[folds!=k,])
    pred<-predict(reg,newdata=movie[folds==k,])
    CV[i]<-CV[i]+ sum((pred-log(BOX[folds==k]))^2)
  }
  CV[i]<-CV[i]/n
}
plot(degree,CV,type='b')

#---------
# Regression splines

# BUDGET
# degree 0
reg<-lm(log(BOX) ~ cut(log(BUDGET),c(min(log(BUDGET)),3,4,4.5,max(log(BUDGET)))),
        data=movie)
plot(log(BUDGET),log(BOX))
xt<-seq(min(BUDGET),max(BUDGET),1)
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)

# degree 1
reg<-lm(log(BOX) ~ bs(log(BUDGET),df=2,degree=1),data=movie)
plot(log(BUDGET),log(BOX))
xt<-seq(min(BUDGET),max(BUDGET),1)
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)

# degree 3
reg<-lm(log(BOX) ~ bs(log(BUDGET),df=7,degree=3),data=movie)
plot(log(BUDGET),log(BOX))
xt<-seq(min(BUDGET),max(BUDGET),1)
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)

# Natural cubic spline
reg<-lm(log(BOX) ~ ns(log(BUDGET),df=5),data=movie)
plot(log(BUDGET),log(BOX))
xt<-seq(min(BUDGET),max(BUDGET),1)
yh<-predict(reg,newdata=data.frame(BUDGET=xt))
lines(log(xt),yh)

# Cross validation

# 10-fold cross-validation
K<-10
folds=sample(1:K,n,replace=TRUE)
df<-c(1,2,3,4,5,7,8)
N<-length(df)
CV<-rep(0,N)
for(i in (1:N)){
  for(k in (1:K)){
    print(c(i,k))
    reg<-lm(log(BOX) ~ ns(log(BUDGET),df=df[i]),data=movie[folds!=k,])
    pred<-predict(reg,newdata=movie[folds==k,])
    CV[i]<-CV[i]+ sum((pred-log(BOX[folds==k]))^2)
  }
  CV[i]<-CV[i]/n
}
plot(df,CV,type='b')


# smoothing slines
plot(log(BUDGET),log(BOX))
ss<-smooth.spline(log(BUDGET),log(BOX))
lines(ss$x,ss$y,col="red",lwd=2)
ss$df


plot(BUZZ,log(BOX))
ss<-smooth.spline(BUZZ,log(BOX))
lines(ss$x,ss$y,col="red",lwd=2)
